
################################################################################
# Corresponding analysis code for:
# Radiomics Detection of Pulmonary Hypertension via Texture-based Assessments 
#   of Cardiac MRI: A Machine-learning Model Comparison
################################################################################


library(MachineShop)
library(recipes)
library(openxlsx)
library(doParallel)

modelInputGrid <- expand.grid(model = c('linear', 'logistic', 'enet', 'ridge', 
                                        'lasso', 'nnet', 'svmRad', 'svmPoly', 
                                        'ada', 'gbm', 'rf', 'mlp'),
                              input = c('corr', 'lincomp', 'pca', 'full'))
modelInputGrid <- modelInputGrid[!(modelInputGrid$model %in% c('linear', 
                                                               'logistic', 
                                                               'nnet') &
                                     modelInputGrid$input == 'full'),]
row.names(modelInputGrid) <- 1:nrow(modelInputGrid)


for (idx in 1:nrow(modelInputGrid)) {
  
  
  ### specify which model/feature selection method will be used
  modelSpec <- modelInputGrid$model[idx]
  inputSpec <- modelInputGrid$input[idx]
  
  ### load data
  # One column of group classifications called "Outcome" and
  #   all other columns containing continuous texture feature values
  # dat <- read.csv('../Data/cleanedDat.csv')
  
  # generate fake data for illustration
  dat <- cbind.data.frame(Outcome = rep(c(1,2), c(40, 42)),
                          matrix(rnorm(82 * 348), nrow = 82, ncol = 348))
  
  ### Outcome variable needs to be a factor
  dat$Outcome <- factor(dat$Outcome)


  ### base recipe
  # Zero-variance filtering
  # normalize all predictors
  base_rec <- recipe(Outcome ~ ., data = dat) %>%
    step_zv(all_predictors()) %>%
    role_case(stratum = Outcome)  %>%
    step_normalize(all_numeric(), -all_outcomes()) 
  
  # different feature selectionn techniques:
  rec_corr <- base_rec %>% step_corr(all_predictors(), -all_outcomes(), 
                                     threshold = .6)
  rec_lincom <- base_rec %>% step_lincomb(all_predictors(), -all_outcomes())
  rec_pca <- base_rec %>% step_pca(all_predictors(), -all_outcomes(), 
                                   threshold = .9)
  
  # set-up modles to run in parallel
  registerDoParallel(cores = detectCores())
  
  ### control settings used for all cases 
  # 5-fold cross-validation with 5 repeats
  control <- CVControl(folds = 5, seed = 123, repeats = 5)
  
  # length of grid search for each tuning parameter
  grid_len <- 5
  
  if (modelSpec == c('linear')) {
    model <- LMModel
  } else if (modelSpec == c('logistic')) {
    model <- GLMModel
  } else if (modelSpec == c('enet')) {
    model <- TunedModel(GLMNetModel, grid = grid_len)
  } else if (modelSpec == c('ridge')) {
    model <- TunedModel(GLMNetModel, grid = grid_len, 
                        fixed = list(alpha = 0))
  } else if (modelSpec == c('lasso')) {
    model <- TunedModel(GLMNetModel, grid = grid_len, 
                        fixed = list(alpha = 1))
  } else if (modelSpec == c('nnet')) {
    model <-  TunedModel(NNetModel, grid = grid_len,
                         fixed = list(MaxNWts = 2000))
  } else if (modelSpec == c('svmRad')) {
    model <-  TunedModel(SVMRadialModel, grid = grid_len)
  } else if (modelSpec == c('svmPoly')) {
    model <-  TunedModel(SVMPolyModel, grid = grid_len)
  } else if (modelSpec == c('ada')) {
    model <-  TunedModel(AdaBoostModel, grid = grid_len)
  } else if (modelSpec == c( 'gbm')) {
    model <-  TunedModel(GBMModel, grid = grid_len)
  } else if (modelSpec == 'rf') {
    model <-  TunedModel(RandomForestModel, grid = grid_len)
  } else if (modelSpec == 'mlp') {
    source('./mlpConfig.R')
    model <- sel_model
  }
  
  if (inputSpec == c('corr')) {
    rec_use <- rec_corr
  } else if (inputSpec == c('lincomp')) {
    rec_use <- rec_lincom
  } else if (inputSpec == c('pca')) {
    rec_use <- rec_pca
  } else if (inputSpec == 'full') {
    rec_use <- base_rec
  }
  
  
  ##############################################################################
  
  # resample performance
  model_res <- resample(rec_use, model = model, control = control)
  
  # obtain the cross-validated AUC
  # Note: fake data was not generated with any differences between groups
  #       expect performance metrics to be poor!
  summary(model_res)
  
  # save results
  saveRDS(model_res, paste0(modelSpec, '_', inputSpec, '.rds'))
}

